# FILE: cygtel.tcl
#
# Configuration feature definitions for the configuration 'cygtel'
#
# Editing instructions begin at "START EDIT HERE" below.

set config [Oc_Config RunPlatform]

if {![string match [string tolower [file rootname [file tail [info script]]]] \
        [$config GetValue platform_name]]} {
    error "Configuration cache file '[info script]' 
sourced by '[$config GetValue platform_name]'"
}

########################################################################
# START EDIT HERE
# In order to properly build, install, and run on your computing 
# platform, the OOMMF software must know certain features of your
# computing environment.  In this file are lines which set the value of 
# certain features of your computing environment.  Each line looks like:
#
# $config SetValue <feature> {<value>}
#
# where each <feature> is the name of some feature of interest,
# and <value> is the value which is assigned to that feature in a
# description of your computing environment.  Your task is to edit
# the values as necessary to properly describe your computing 
# environment.
#
# The character '#' at the beginning of a line is a comment character.
# It causes the contents of that line to be ignored.  To select
# among lines providing alternative values for a feature, uncomment the
# line containing the proper value.
#
# The features in this file are divided into three sections.  The first
# section (REQUIRED CONFIGURATION) includes features which require you 
# to provide a value.  The second section (OPTIONAL CONFIGURATION)
# includes features which have usable default values, but which you
# may wish to customize.  The third section (ADVANCED CONFIGURATION)
# contains features which you probably do not need or want to change
# without a good reason.
########################################################################
# REQUIRED CONFIGURATION

# NOTE: The rest of the REQUIRED CONFIGURATION is required only
# for building OOMMF software from source code.  If you downloaded
# a distribution with pre-compiled executables, no more configuration
# is required.

# Set the feature 'program_compiler_c++' to the program to run on this 
# platform to compile source code files written in the language C++ into 
# object files.  Select from the choices below.  If the compiler is not 
# in your path, be sure to use the whole pathname.  Also include any 
# options required to instruct your compiler to only compile, not link.  
#
# If your compiler is not listed below, additional features will
# have to be added in the ADVANCED CONFIGURATION section below to 
# describe to the OOMMF software how to operate your compiler.  Send
# e-mail to the OOMMF developers for assistance.
#
# Microsoft Visual C++
# <URL:http://www.microsoft.com/visualc/>
# $config SetValue program_compiler_c++ {cl /nologo /c}
#
# The GNU C++ compiler 'g++'
# <URL:http://www.gnu.org/software/gcc/gcc.html>
# <URL:http://egcs.cygnus.com/>
$config SetValue program_compiler_c++ {g++ -c}


########################################################################
# OPTIONAL CONFIGURATION

# Set the feature 'path_directory_temporary' to the name of an existing 
# directory on your computer in which OOMMF software should write 
# temporary files.  All OOMMF users must have write access to this 
# directory.
#
# $config SetValue path_directory_temporary {C:\temp}
# $config SetValue path_directory_temporary {C:\}

########################################################################
# SUPPORT PROCEDURES
#
# Routine to guess the CPU using /proc/cpuinfo provided by Cygwin kernel
proc GuessCPU {} {
    set guess unknown
    catch {
	set fptr [open "|cat /proc/cpuinfo" r]
	set cpustr [read $fptr]
	close $fptr
	# Look at first processor info only
	set tmplst [split $cpustr "\n"]
	set record_end [lsearch $tmplst {}]
	if {$record_end>0} {
	    set tmplst [lreplace $tmplst $record_end end]
	}
	set cpustr [join $tmplst ":"]
	foreach {x y} [split $cpustr ":"] {
	    set cpuinfo([string trim $x]) [string tolower [string trim $y]]
	}
	# Try to guess processor based on Intel family + model info
	if {[string match "*intel*" $cpuinfo(vendor_id)]} {
	    if {$cpuinfo(cpu family) >= 15} {
		set guess pentium4
               if {[lsearch -exact $cpuinfo(flags) lm]>=0} {
                  set guess nocona  ;# nocona is improved version of
                  ## pentium 4 with 64-bit extensions, i.e., x86-64.
                  ## Good guess that nocona can also be identified as
                  ## cpu family 15, model 4.  The flags value "lm"
                  ## means "long mode", i.e., 64-bit.
               }
	    } elseif {$cpuinfo(cpu family)==6} {
		if {$cpuinfo(model)<3} {
		    set guess pentiumpro
		} elseif {$cpuinfo(model)<7} {
		    set guess pentium2
		} else {
		    # model number >=8
		    set guess pentium3
		}
	    }
	}
	if {[string match "unknown" $guess]} {
	    # Otherwise, try to match against "model name"
	    switch -regexp -- $cpuinfo(model name) {
		{^386}                    { set guess 386 }
		{^486}                    { set guess 486 }
		{pentium(\(r\)|) (iv|4)}  { set guess pentium4 }
		{pentium(\(r\)|) (iii|3)} { set guess pentium3 }
		{pentium(\(r\)|) (ii|2)}  { set guess pentium2 }
		{pentium(\(r\)|) pro}     { set guess pentiumpro }
		{pentium}                 { set guess pentium }
		{overdrive podp5v83}      { set guess pentium }
		{athlon(\(tm\)|) mp}      { set guess athlon-mp }
		{athlon(\(tm\)|) xp}      { set guess athlon-xp }
		{athlon(\(tm\)|) (iv|4)}  { set guess athlon-4 }
		{athlon(\(tm\)|) (tbird)} { set guess athlon-tbird }
		{athlon}                  { set guess athlon }
		{opteron}                 { set guess opteron }
		{^k6}                     { set guess k6 }
		{^k5}                     { set guess k5 }
	    }
	}
    }
    return $guess
}

# Routine to guess the gcc version
proc GuessGccVersion { gcc } {
    set guess {}
    catch {
	set fptr [open "|$gcc --version" r]
	set verstr [read $fptr]
	close $fptr
	regexp -- {[0-9][0-9.]+} $verstr guess
    }
    return [split $guess "."]
}

########################################################################
# ADVANCED CONFIGURATION

# The absolute, native filename of the null device
$config SetValue path_device_null {nul:}

# Compiler option processing...
set ccbasename [file tail [lindex [$config GetValue program_compiler_c++] 0]]
if {[string match g++ $ccbasename]} {
    # ...for GNU g++ C++ compiler
    set gccversion [GuessGccVersion \
			[lindex [$config GetValue program_compiler_c++] 0]]

    # Optimization options
    # set opts [list -O0 -fno-inline -ffloat-store]  ;# No optimization
    # set opts [list -O%s]                      ;# Minimal optimization
    set opts [list -O3 -ffast-math -fstrict-aliasing]
    # You can also try adding -malign-double.  See next block for
    # comments concerning -fomit-frame-pointer.  Processor
    # architecture selection is farther down.

    # Frame pointer: Some versions of gcc don't handle exceptions
    # properly w/o frame-pointers.  This typically manifests as
    # Oxs dying without an error message while loading a MIF file.
    # Interestingly, including -momit-leaf-frame-pointer appears
    # to work around this problem, at least on some systems.  YMMV;
    # Comment this out if the aforementioned problem occurs.
    lappend opts -fomit-frame-pointer -momit-leaf-frame-pointer

    # The next block sets cpu-specific options, based on a guess (see
    # GuessCPU proc defined at the top of this file) of the current
    # CPU and gcc version.  If this guess is wrong, or you want to
    # build for a generic CPU, unset cpuopts below.  NOTE: This is a
    # first try, so it is most certainly broken!
    set cputype [GuessCPU]
    if {[llength $gccversion]>1} {
	# Set cpu-dependent compiler options
	set verA [lindex $gccversion 0]
	set verB [lindex $gccversion 1]
	if {$verA==2 && $verB>=95} {
	    switch -glob -- $cputype {
		k6       { set cpuopts [list -march=k6] }
		pentium  { set cpuopts [list -march=pentium] }
		athlon*  -
		opteron  -
                nocona   -
		pentium* { set cpuopts [list -march=pentiumpro] }
	    }
	} elseif {$verA==3 && $verB<1} {
	    switch -glob -- $cputype {
		pentium  { set cpuopts [list -march=pentium] }
                nocona   -
		pentium* { set cpuopts [list -march=pentiumpro] }
		k6       { set cpuopts [list -march=k6] }
		opteron  -
		athlon*  { set cpuopts [list -march=athlon] }
	    }
	} elseif {$verA>=3} {
	    switch -exact -- $cputype {
		pentium    { set cpuopts [list -march=pentium] }
		pentiumpro { set cpuopts [list -march=pentiumpro] }
		pentium2   { set cpuopts [list -march=pentium2] }
		pentium3   { set cpuopts [list -march=pentium3] }
		pentium4   { set cpuopts \
				 [list -march=pentium4 -msse2 -mfpmath=sse] }
		nocona     { set cpuopts \
				 [list -march=nocona -msse3 -mfpmath=sse] }
		k6         { set cpuopts [list -march=k6] }
		athlon     { set cpuopts [list -march=athlon] }
		athlon-tbird { set cpuopts [list -march=athlon-tbird] }
		athlon-4   { set cpuopts [list -march=athlon-4] }
		athlon-xp  { set cpuopts [list -march=athlon-xp] }
		athlon-mp  { set cpuopts [list -march=athlon-mp] }
		opteron    { set cpuopts \
				 [list -march=athlon -msse2 -mfpmath=sse] }
	    }
	}
    }
    # You can override the GuessCPU results by directly setting or
    # unsetting the cpuopts variable, e.g.,
    #
    #    unset cpuopts
    #    set cpuopts [list -march=athlon]
    #
    if {[info exists cpuopts] && [llength $cpuopts]>0} {
	set opts [concat $opts $cpuopts]
    }
    $config SetValue program_compiler_c++_option_opt "format \"$opts\""
    # NOTE: If you want good performance, be sure to edit ../options.tcl
    #  or ../local/options.tcl to include the line
    #    Oc_Option Add * Platform cflags {-def NDEBUG}
    #  so that the NDEBUG symbol is defined during compile.
    $config SetValue program_compiler_c++_option_out {format "-o \"%s\""}
    $config SetValue program_compiler_c++_option_src {format \"%s\"}
    $config SetValue program_compiler_c++_option_inc {format "\"-I%s\""}
 
    # Compiler warnings:
    # Omitted: -Wredundant-decls -Wshadow -Wcast-align
    # I would also like to use -Wcast-qual, but casting away const is
    # needed on some occasions to provide "conceptual const" functions in
    # place of "bitwise const"; cf. p76-78 of Meyer's book, "Effective C++."
    #
    # NOTE: -Wno-uninitialized is required after -Wall by gcc 2.8+ because
    # of an apparent bug.  -Winline is out because of failures in the STL.
    # Depending on the gcc version, the following options may also be
    # available:     -Wbad-function-cast     -Wstrict-prototypes
    #                -Wmissing-declarations  -Wnested-externs
    $config SetValue program_compiler_c++_option_warn {format "-Wall \
        -W -Wpointer-arith -Wwrite-strings -Wmissing-prototypes \
        -Woverloaded-virtual -Wsynth -Werror"}
    $config SetValue program_compiler_c++_option_debug {format "-g"}
    $config SetValue program_compiler_c++_option_def {format "\"-D%s\""}

    # Wide floating point type.
    # NOTE: On the Linux/x86+gcc platform, "long double" provides
    # somewhat better precision than "double", but at a cost of
    # increased memory usage and a decrease in speed.  (At this writing,
    # long double takes 12 bytes of storage as opposed to 8 for double,
    # but provides the x86 native floating point format having approx.
    # 19 decimal digits precision as opposed to 16 for double.)
    # Default is "double".
    $config SetValue program_compiler_c++_typedef_realwide "double"

    # Directories to exclude from explicit include search path, i.e.,
    # the -I list.  Some versions of gcc complain if "system" directories
    # appear in the -I list.
    $config SetValue \
    	program_compiler_c++_system_include_path [list /usr/include]

    # Other compiler properties
    $config SetValue \
            program_compiler_c++_property_optimization_breaks_varargs 0

    # Missing function prototypes
    $config SetValue program_compiler_c++_prototype_supply_nice \
      {extern "C" int nice(int);}
}

# The program to run on this platform to link together object files and
# library files to create an executable binary.
set lbasename $ccbasename
if {[string match g++ $ccbasename]} {
    # ...for GNU g++ as linker
    $config SetValue program_linker [list $lbasename]
    $config SetValue program_linker_option_obj {format \"%s\"}
    $config SetValue program_linker_option_out {format "-o \"%s\""}
    $config SetValue program_linker_option_lib {format \"%s\"}
    $config SetValue program_linker_uses_-L-l {1}
    proc fubar { subsystem } { return "-m[string tolower $subsystem]" }
    $config SetValue program_linker_option_sub {fubar}
}
unset lbasename

# The program to run on this platform to create a single library file out
# of many object files.
if {[string match g++ $ccbasename]} {
    $config SetValue program_libmaker {ar crs}
    $config SetValue program_libmaker_option_obj {format \"%s\"}
    $config SetValue program_libmaker_option_out {format \"%s\"}
}
unset ccbasename

# Name formats on generated files
$config SetValue script_filename_object {format %s.o}
$config SetValue script_filename_static_library {format lib%s.a}
$config SetValue script_filename_executable {format %s.exe}

# Some Cygwin distributions appear to have a bad TCL_RANLIB entry
# Check it.
if {[catch {$config GetValue TCL_RANLIB} ranlib]
    || ![llength [auto_execok $ranlib]]} {
    $config SetValue TCL_RANLIB ranlib
}
unset ranlib

# If we are building with a Windows build of Tcl/Tk,
# then tclConfig.sh and tkConfig.sh probably won't
# be available.
if {[catch {$config GetValue TCL_LIBS}]} {
    $config SetValue TCL_LIBS {-lm}
}
if {[catch {$config GetValue TK_LIBS}]} {
    $config SetValue TK_LIBS {-lm}
}

# String to pass to linker to pick up the Tcl and Tk libraries from
# the install directory.
catch {$config SetValue TCL_LIB_SPEC
    [list -L[file join [$config GetValue TCL_EXEC_PREFIX] lib]
     -ltcl[$config GetValue
           TCL_MAJOR_VERSION][$config GetValue TCL_MINOR_VERSION]]}
catch {$config SetValue TK_LIB_SPEC 
    [list -L[file join [$config GetValue TCL_EXEC_PREFIX] lib] 
     -ltk[$config GetValue 
          TK_MAJOR_VERSION][$config GetValue TK_MINOR_VERSION]]}

# Additional libraries to use when linking against TCL, Tk
# -luser32 is needed if SUBSYSTEM==WINDOWS in ext/messages.cc
# to provide MessageBeep (and maybe MessageBox?).
#$config SetValue TCL_LIBS {-luser32}
#$config SetValue TK_LIBS {-luser32}

# Workaround for bad lib path in patched cygwin-b20.1 (2000-03-20).  On the
# stock b20.1 release this should have no effect.
#set goodpath "/cygnus/cygwin-b20/H-i586-cygwin32/lib/gcc-lib/i586-cygwin32/2.95.2"
#$config SetValue TK_LIB_SPEC [concat -L$goodpath [$config GetValue TK_LIB_SPEC]]
#$config SetValue TCL_LIB_SPEC [concat -L$goodpath [$config GetValue TCL_LIB_SPEC]]
#unset goodpath

# Workaround for bad TK_BUILD_INCLUDES in tkConfig.sh, Cygwin Tcl/Tk
# package 20001125-1, concurrent with Cygwin DLL release 1.3.13-2.
# Update 16-Jan-2003: Build of win side of Tcl/Tk 8.4.1, under Cygwin,
#  does not set TK_XINCLUDES in tkConfig.sh, so added catch around
#  GetValue TK_XINCLUDES to handle this case.
set oldpath {}
#catch {set oldpath [$config GetValue TK_XINCLUDES]}
$config SetValue TK_XINCLUDES [concat $oldpath "-I/usr/X11R6/include"]
unset oldpath

$config SetValue TK_LIBS {}

# Directory holding init.tcl.  Use this if environment variable
# TCL_LIBRARY is not set.
catch {$config SetValue TCL_LIBRARY \
           [file join [$config GetValue TCL_PREFIX] \
                lib tcl[$config GetValue TCL_VERSION]]}

########################################################################
unset config
